#! /usr/bin/python3
import sys, math
import matplotlib.pyplot as plt
import numpy as np
from scipy.spatial import ConvexHull
import matplotlib
matplotlib.use('Agg')

def find_external_tangent_points(x1, y1, r1, x2, y2, r2):
    # Tính khoảng cách giữa hai tâm
    d = math.sqrt((x2 - x1)**2 + (y2 - y1)**2)

    # Kiểm tra nếu hai đường tròn nằm ngoài nhau
    if d <= abs(r1 - r2):
        return None
    
    # Góc của đường nối tâm
    base_angle = math.atan2(y2 - y1, x2 - x1)

    # Góc của tiếp tuyến ngoài
    angle = math.acos((r1 + r2) / d)

    # Các tiếp tuyến ngoài
    tangent_points = []
    for delta in [angle, -angle]:
        angle_t = base_angle + delta
        x_t1 = x1 + r1 * math.cos(angle_t)
        y_t1 = y1 + r1 * math.sin(angle_t)
        x_t2 = x2 + r2 * math.cos(angle_t)
        y_t2 = y2 + r2 * math.sin(angle_t)
        tangent_points.append((x_t1, y_t1))
        tangent_points.append((x_t2, y_t2))

    return tangent_points

def find_internal_tangent_points(x1, y1, r1, x2, y2, r2):
    # Tính khoảng cách giữa hai tâm
    d = math.sqrt((x2 - x1)**2 + (y2 - y1)**2)

    # Kiểm tra nếu hai đường tròn có thể có tiếp tuyến trong
    if d <= abs(r1 - r2):
        return None

    # Góc của đường nối tâm
    base_angle = math.atan2(y2 - y1, x2 - x1)

    # Góc của tiếp tuyến trong
    angle = math.acos((r1 - r2) / d)

    # Các tiếp tuyến trong
    tangent_points = []
    for delta in [angle, -angle]:
        angle_t = base_angle + delta
        x_t1 = x1 + r1 * math.cos(angle_t)
        y_t1 = y1 + r1 * math.sin(angle_t)
        x_t2 = x2 - r2 * math.cos(angle_t)
        y_t2 = y2 - r2 * math.sin(angle_t)
        tangent_points.append((x_t1, y_t1))
        tangent_points.append((x_t2, y_t2))

    return tangent_points


def find_convex_hull(points):
    # Tìm điểm gốc (phần tử có tung độ thấp nhất, nếu có nhiều thì chọn có hoành độ thấp nhất)
    start_point = min(points, key=lambda p: (p[1], p[0]))
    
    # Sắp xếp các điểm theo góc tạo với điểm gốc
    sorted_points = sorted(points, key=lambda p: (math.atan2(p[1] - start_point[1], p[0] - start_point[0])))
    
    # Xây dựng đa giác lồi từ các điểm đã sắp xếp
    convex_hull = []
    for point in sorted_points:
        while len(convex_hull) >= 2 and not is_turn_left(convex_hull[-2], convex_hull[-1], point):
            convex_hull.pop()
        convex_hull.append(point)
    
    return convex_hull

def is_turn_left(p1, p2, p3):
    # Kiểm tra xem có phải quay trái không
    return (p2[0] - p1[0]) * (p3[1] - p2[1]) > (p2[1] - p1[1]) * (p3[0] - p2[0])

def plot_non_intersecting_lines(points, circles):
    # Tính toán convex hull
#    hull = ConvexHull(points)
    
    # Lấy các điểm trên vỏ lồi theo thứ tự góc
#    hull_points = points[hull.vertices]
#    print (hull_points)
#    sys.exit(0)
    hull_points = find_convex_hull(points)
    hull_points = np.array(hull_points)
    
#    print (hull_points)
#    sys.exit(0)
    fig, ax = plt.subplots()
    for key, circle in circles.items():
       circle = plt.Circle((circle[0], circle[1]), circle[2], color='r', fill=False)
       ax.add_artist(circle)
    # Vẽ các điểm
    plt.plot(points[:,0], points[:,1], 'o')
    
    # Vẽ đường nối của convex hull
    for i in range(len(hull_points)):
        plt.plot([hull_points[i, 0], hull_points[(i+1) % len(hull_points), 0]],
                 [hull_points[i, 1], hull_points[(i+1) % len(hull_points), 1]], 'k-')
    #
    ax.set_xlim(110, 118)
    ax.set_ylim(17, 27)
    plt.gca().set_aspect('equal', adjustable='box')
    plt.grid(True)
    plt.title('Non-Intersecting Lines Example')
    plt.savefig('test.png')

circles = {'R0':(111.9, 18.900000000000002, 0.9009009009009009),
           'R1':(111.9, 20.400000000000002, 0.9009009009009009),
           'R2':(112.2, 21.8, 1.3513513513513513),
           'R3':(115.0, 24.5, 1.3513513513513513)
          }

circles = {
           'R0':(111.9, 20.400000000000002, 0.9009009009009009),
           'R1':(115.0, 24.5, 1.3513513513513513)
          }

k = 0
ncircle = len(circles)
points = []
for icircle in range(1,ncircle):
   circle1 = circles['R'+'%1.1d'%(icircle-1)]
   x1, y1, r1 = circle1[0], circle1[1], circle1[2]
   circle2 = circles['R'+'%1.1d'%(icircle)]
   x2, y2, r2 = circle2[0], circle2[1], circle2[2]
   print (x1, y1, r1, x2, y2, r2)
   tangent_points1, tangent_points2, tangent_points3, tangent_points4 = find_external_tangent_points(x1, y1, r1, x2, y2, r2)
#   print (tangent_points)
#   sys.exit(0)
   #tangent_points1, tangent_points2, tangent_points3, tangent_points4 = find_tangents(circle1[0], circle1[1], r1, circle2[0], circle2[1], r2)
   #tangent_points1, tangent_points2, tangent_points3, tangent_points4 = tangent_points(circle1, r1, circle2, r2)
   points += ((tangent_points1[0], tangent_points1[1]), (tangent_points2[0], tangent_points2[1]), (tangent_points3[0], tangent_points3[1]), (tangent_points4[0], tangent_points4[1]))
#print (tangent_points1, tangent_points2, tangent_points3, tangent_points4)
#print (len(points))


#sys.exit(0)
# Example usage:
# Các điểm của bạn
#points = [
#    (110.9990990990991, 18.900000000000002), (112.80090090090091, 18.900000000000002),
#    (112.80090090090091, 20.400000000000002), (110.9990990990991, 20.400000000000002),
#    (111.00444076170616, 20.30204039633295), (112.67678522367277, 19.943680868768677),
#    (113.54333885744077, 21.946939405500583), (111.03482216449086, 22.48447869684699),
#    (112.20130500081218, 23.15135072123117), (113.55050500036101, 21.752180351328683),
#    (115.00130500081218, 25.85135072123117), (116.35050500036101, 24.452180351328682)
#]
points = np.array(points)
print (points)
plot_non_intersecting_lines(points, circles)
